use axum::{
    async_trait,
    extract::{FromRef, FromRequestParts},
    http::{request::Parts, StatusCode},
    RequestPartsExt, TypedHeader,
};
use headers::Cookie;
use tracing::error;

use crate::{model::user::User, state::AppState, token::Claims};

pub struct ExtractUserToken(pub Option<User>);

pub const TOKEN_COOKIE_NAME: &'static str = "imgboard-token";

#[async_trait]
impl<S> FromRequestParts<S> for ExtractUserToken
where
    AppState: FromRef<S>,
    S: Send + Sync,
{
    type Rejection = (StatusCode, &'static str);

    async fn from_request_parts(parts: &mut Parts, state: &S) -> Result<Self, Self::Rejection> {
        let state = match parts.extract_with_state::<AppState, _>(state).await {
            Ok(s) => s,
            Err(err) => {
                error!("Getting state: {}", err);
                return Err((StatusCode::INTERNAL_SERVER_ERROR, "Internal server error"));
            }
        };

        let cookie = match parts.extract::<TypedHeader<Cookie>>().await {
            Ok(cookie) => cookie,
            Err(err) => {
                error!("Getting cookie header: {}", err);
                return Ok(ExtractUserToken(None));
            }
        };

        match cookie.get(TOKEN_COOKIE_NAME) {
            Some(token) => {
                let claims = Claims::decode(token, &state.decoding_key).map_err(|e| {
                    error!("Decoding token claims: {}", e);
                    (StatusCode::INTERNAL_SERVER_ERROR, "Internal server error")
                })?;

                match sqlx::query_as!(User, r#"SELECT * FROM users WHERE id = $1"#, claims.uid)
                    .fetch_one(&state.pool)
                    .await
                {
                    Ok(u) => Ok(ExtractUserToken(Some(u))),
                    Err(err) => {
                        error!("Getting user from database: {}", err);
                        Err((StatusCode::INTERNAL_SERVER_ERROR, "Internal server error"))
                    }
                }
            }
            None => Ok(ExtractUserToken(None)),
        }
    }
}
