use axum::{
    http::{header, StatusCode},
    response::IntoResponse,
};
use eyre::{Context, Result};
use handlebars::{Handlebars, RenderError};
use rust_embed::RustEmbed;
use serde::Serialize;
use tracing::{error, info};

use crate::model::user::User;

#[derive(RustEmbed)]
#[folder = "templates/"]
struct Templates;

pub fn handlebars(dev_mode: bool) -> Result<Handlebars<'static>> {
    let mut hbs = Handlebars::new();

    if dev_mode {
        info!("Reading templates from file system, as dev mode is enabled");

        hbs.set_dev_mode(true);
        hbs.register_templates_directory("", "templates/")
            .wrap_err("registering templates from directory")?;
    } else {
        hbs.register_embed_templates::<Templates>()
            .wrap_err("registering templates from embed")?;
    }

    Ok(hbs)
}

pub struct Page(Result<String, RenderError>);

impl Page {
    pub fn new(res: Result<String, RenderError>) -> Self {
        Self(res)
    }
}

impl IntoResponse for Page {
    fn into_response(self) -> axum::response::Response {
        match self.0 {
            Ok(s) => ([(header::CONTENT_TYPE, "text/html; charset=utf-8")], s).into_response(),
            Err(err) => {
                error!("Error rendering page: {}", err);
                (StatusCode::INTERNAL_SERVER_ERROR, "Internal server error").into_response()
            }
        }
    }
}

#[derive(Debug, Default, Serialize)]
pub struct PageData {
    pub user: Option<User>,

    // An optional flash message. Not all templates will handle this.
    pub message: Option<String>,
}
